@@ -1,4 +1,4 @@
-from Bubot.Helpers.ExtException import ExtException, KeyNotFound
+from Bubot.Helpers.ExtException import ExtException, KeyNotFound, UserError
 from Bubot.Helpers.ArrayHelper import ArrayHelper
 from Bubot.Helpers.Helper import Helper
 from Bubot.Helpers.Action import Action
