@@ -3,18 +3,21 @@ from typing import TYPE_CHECKING
 from uuid import uuid4
 
 from blockly_executor import ExtException
-from blockly_executor.core.exceptions import LimitCommand, StepForward, DeferredOperation, ReturnFromFunction, \
-    WorkspaceNotFound
+from blockly_executor.core.exceptions import StepForward, DeferredOperation, ServiceException, ErrorInBlock
 
 if TYPE_CHECKING:
     from .executor import BlocklyExecutor
+    from .workspace import Workspace
 
 
 class Block:
     ns = {'b': 'https://developers.google.com/blockly/xml'}
+    _result = '_result'
+    statement_inputs = None
 
     def __init__(self, executor: 'BlocklyExecutor', **kwargs):
         self.executor: BlocklyExecutor = executor
+        self.workspace: Workspace = executor.workspace
         self.block_id = None
         self.logger = kwargs.get('logger', logging)
 
@@ -26,20 +29,20 @@ class Block:
         return self
 
     def _check_step(self, context, block_context):
-        if self.executor.debug_mode:
+        if context.debug_mode:
             if context.is_next_step:
-                if self.executor.debug_mode == 'step':
-                    raise StepForward(self.block_id, context, block_context, self.executor.algorithm)
+                if context.debug_mode == 'step':
+                    raise StepForward(self.block_id, context, block_context, self.executor.workspace.name)
                 else:
                     context.current_block = self.block_id
-                    context.current_algorithm = self.executor.algorithm
-            if self.block_id == context.current_block and context.current_algorithm == self.executor.algorithm:
+                    context.current_workspace = self.executor.workspace.name
+            if self.block_id == context.current_block and context.current_workspace == self.executor.workspace.name:
                 if self.executor.current_algorithm_breakpoints:
                     if self.block_id in self.executor.current_algorithm_breakpoints and context.is_next_step:
-                        raise StepForward(self.block_id, context, block_context, self.executor.algorithm)
-                if self.executor.debug_mode == 'step':
+                        raise StepForward(self.block_id, context, block_context, self.executor.workspace.name)
+                if context.debug_mode == 'step':
                     if context.is_next_step is False:  # встаем на этом шаге
-                        raise StepForward(self.block_id, context, block_context, self.executor.algorithm)
+                        raise StepForward(self.block_id, context, block_context, self.executor.workspace.name)
                     else:
                         context.is_next_step = True  # встаем на следующем шаге
 
@@ -51,15 +54,6 @@ class Block:
         # context.init_block_context(block_context, self.block_id, path)
         pass
 
-    @classmethod
-    def get_child_block(cls, node):
-        child = None
-        if node:
-            child = node.find('./b:block', cls.ns)
-            if child is None:
-                child = node.find('./b:shadow', cls.ns)
-        return child
-
     async def execute_all_next(self, node, path, context, block_context, statement=False):
         statement_number = 0
         child_context = context.get_child_context(block_context)
@@ -75,40 +69,14 @@ class Block:
             else:
                 _child_context = child_context
 
-            child = self.get_child_block(node)
+            child = self.executor.workspace.get_child_block(node)
 
             next_node = None
 
             if child is not None:
-                next_node = self.get_next_statement(child) if statement else None
-                block_subtype = child.get('type')
+                next_node = self.workspace.get_next_statement(child) if statement else None
                 if '__result' not in _child_context:
-                    # path = f'{path}.{block_subtype}'
-                    _class = self.executor.get_block_class(block_subtype)
-                    try:
-                        result = await _class(self.executor, logger=self.logger).execute(
-                            child, block_context['__path'], context, _child_context)
-                    except LimitCommand as err:
-                        raise err from err
-                    except StepForward as err:
-                        raise err from err
-                    except ReturnFromFunction as err:
-                        raise err from err
-                    except WorkspaceNotFound as err:
-                        raise err from err
-                    except DeferredOperation as err:
-                        raise err from err
-                    except ExtException as err:
-                        raise ExtException(parent=err)
-                    except Exception as err:
-                        raise ExtException(message='Ошибка в блоке',
-                                           detail=f'{block_subtype}: {err}',
-                                           parent=err,
-                                           dump=dict(
-                                               block=self.block_id,
-                                               algorithm=self.executor.algorithm
-                                           ),
-                                           skip_traceback=1)
+                    result = await self.execute_child_block(child, block_context['__path'], context, _child_context)
                 else:
                     self.logger.debug(f'{child.get("id")} skip')
             else:
@@ -128,12 +96,32 @@ class Block:
         return await self.execute_all_next(node, path, context, block_context)
         pass
 
+    async def execute_child_block(self, child_node, path, context, child_block_context):
+        block_type = child_node.get('type')
+        _class = self.executor.get_block_class(block_type)
+        try:
+            return await _class(self.executor, logger=self.logger).execute(
+                child_node, path, context, child_block_context)
+        except (ServiceException, ErrorInBlock) as err:
+            raise err from err
+        except ExtException as err:
+            raise err from err
+        except Exception as err:
+            raise ExtException(message='Ошибка в блоке',
+                               detail=f'{block_type}: {err}',
+                               parent=err,
+                               dump=dict(
+                                   block=self.block_id,
+                                   algorithm=self.executor.workspace.name
+                               ),
+                               skip_traceback=1)
+
     async def execute(self, node, path, context, block_context):
         try:
             await self._before_execute(node, path, context, block_context)
             self.logger.debug(f'{self.full_name} begin execute id="{self.block_id}"')
             return await self._execute(node, path, context, block_context)
-        except (LimitCommand, StepForward, ReturnFromFunction, DeferredOperation) as err:
+        except ServiceException as err:
             raise err from err
 
     @property
@@ -186,36 +174,6 @@ class Block:
             detail=f'block {self.full_name} result type {_status}',
         )
 
-    @classmethod
-    def get_next_statement(cls, node):
-        return node.find(f"./b:next", cls.ns)
-
-    @classmethod
-    def _get_mutation(cls, node, mutation_name, default=None):
-        mutation = node.find(f'./b:mutation', cls.ns)
-        if mutation is not None:
-            return mutation.get(mutation_name, default)
-        return default
-
-    # async def _calc_all_mutation(self, node, path, context, block_context, mutation_name):
-    #     mutation_count = self._get_mutation(node, mutation_name)
-    #     if mutation_count:
-    #         for j in range(mutation_count):
-    #             # рассчитываем все мутации
-    #             _key = f'{mutation_name}{j}'
-    #             complete = _key in block_context
-    #             if not complete:
-    #                 result = await self._calc_mutation(node, path, context, block_context, j)
-    #                 block_context[_key] = result
-    #
-    # async def _calc_mutation(self, node, path, context, block_context, _key):
-    #     raise NotImplemented()
-
-    # @classmethod
-    # def set_thread_variable(cls, context, name, value):
-    #     context.debug['__thread_vars'][name] = value
-    #     cls.set_variable(context, name, value)
-
     @classmethod
     def defined_variable(cls, context, name):
         try:
