@@ -3,6 +3,8 @@ from blockly_executor.core.exceptions import LimitCommand, DeferredOperation
 
 
 class MultiThreadLoop(Block):
+    statement_inputs = ['STACK']
+
     def __init__(self, executor, **kwargs):
         super().__init__(executor, **kwargs)
 
@@ -30,9 +32,8 @@ class MultiThreadLoop(Block):
 
     async def _execute_items(self, node, path, context, block_context):
         self._on_before_loop(node, path, context, block_context)
-        var_name = node[0].text
         try:
-            node_loop = self.get_node_statement(node)  # алгоритм внутри цикла
+            node_loop = self.workspace.find_statement_by_name(node, self.statement_inputs[0])  # алгоритм внутри цикла
         except IndexError:  # не заполнено содержимое
             return
         if 'items' not in block_context:
@@ -92,6 +93,3 @@ class MultiThreadLoop(Block):
         await self._execute_items(node, path, context, block_context)
         self.executor.multi_thread_mode = False
 
-    @classmethod
-    def get_node_statement(cls, node):
-        return node.find("./b:statement", cls.ns)
