@@ -1,3 +1,6 @@
+from uuid import uuid4
+from blockly_executor import ExtException
+from blockly_executor import Action
 from blockly_executor import Helper, ArrayHelper
 from blockly_executor.core.exceptions import LimitCommand
 from .context_props import ContextProps
@@ -6,18 +9,9 @@ from .context_props import ContextProps
 class Context(ContextProps):
     def __init__(self):
         super().__init__()
+        self.uid = None
         self.data = {}
-        # self.params = {}  # параметры операции передающиеся между вызовами
-        # self.variables = {}  # значения переменных
-        # self.block_context = {'__thread_vars': {}}  # контекст текущего блока, показывется при отладке
-        # self.operation = {}  # контекст операции
-        # self.deferred = []
-        self.protocol = None
-
-        self.algorithm = None
-        self.debug_mode = None
-        self.current_block = None
-        self.current_algorithm = None
+        self.action = Action()
         self.current_thread = None
 
         self.is_deferred = False
@@ -25,78 +19,55 @@ class Context(ContextProps):
         self.deferred_result = None
         self.limit_commands = 25
 
-    def _init_from_dict(self, data):
-        # self.variables = data.get('variables', {})
-        # self.block_context = data.get('block_context', {'__thread_vars': {}})
-        # self.operation = data.get('operation', {
-        #     'status': 'run',
-        #     'result': None,
-        #     'commands': []
-        # })
-        # self.deferred = data.get('deferred', [])
-        pass
-
     @classmethod
-    def init(cls, *, debug_mode=None, protocol=None, current_block=None, current_algorithm=None,
-             algorithm=None, data=None, **kwargs):
-        # if not operation_id:
-        #     operation_id = str(uuid4())
+    async def init(cls, *, uid=None, debug_mode=None, current_block=None, current_workspace=None, workspace_name=None, data=None,
+                   **kwargs):
         self = cls()
-        if protocol:
-            self.data = protocol.blockly_context
-        else:
-            self.data = data if data else {}
-        self.protocol = protocol
+        self.uid = uid if uid else str(uuid4())
+        self.data = data if data else {}
         self.debug_mode = debug_mode
         self.current_block = current_block
-        self.current_algorithm = current_algorithm if current_algorithm else algorithm
-        self.algorithm = algorithm
+        self.current_workspace = current_workspace if current_workspace else workspace_name
+        self.workspace_name = workspace_name
         self.is_next_step = True if not current_block and self.debug_mode else None
-
-        # if not data:
-        #     data = {}
-        # self._init_from_dict(data)
-        #
-        # self.params = params if params else {}
-
-        # self.operation['begin'] = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
-
         return self
 
     def init_deferred(self, block_context):
+        """
+        Контекст для отложенного исполнения
+
+        :param block_context:
+        :return:
+        """
         _self = self.__class__()
-        # _self.params = self.params
         _self.block_context = block_context['block_context']
         _self.is_deferred = True
         _self.variables = self.variables
-        # _self.operation = self.operation
         _self.deferred = self.deferred
         return _self
 
-    def init_nested(self, block_context):
+    def init_nested(self, block_context, workspace_name):
+        """
+        Контекст для запуска вложенного алгоритма
+        :param workspace_name:
+        :param block_context:
+        :return:
+        """
         _self = self.__class__()
-        # _self.params = self.params
-        _self.block_context = block_context.get('_child_block_context', {})
-        _self.is_deferred = True
-        _self.variables = block_context.get('_child_variables', {})
-        # _self.operation = self.operation
-        _self.deferred = self.deferred
-        _self.protocol = self.protocol
+        _self.data = block_context.get('_child', {})
+        _self.workspace_name = workspace_name
+
+        _self.is_deferred = self.is_deferred
         _self.debug_mode = self.debug_mode
         _self.current_block = self.current_block
-        _self.current_algorithm = self.current_algorithm
-        # _self.algorithm = algorithm
+        _self.current_workspace = self.current_workspace
+        _self.is_next_step = self.is_next_step
         return _self
 
-    @property
-    def operation_id(self):
-        return self.operation.get('operation_id')
-
     def to_parallel_dict(self):
         return dict(
             variables=self.variables,
             block_context=self.block_context,
-            operation=self.operation,
         )
 
     def to_dict(self):
@@ -104,22 +75,21 @@ class Context(ContextProps):
 
     def to_result(self, debug_mode):
         res = dict(
+            uid=self.uid,
             result=self.result,
             status=self.status,
             commands=self.commands,
         )
-        if self.protocol:
-            res['protocol'] = self.protocol.uuid
         if debug_mode:
-            res['variables'] = self.current_variables
+            res['current_variables'] = self.current_variables
             res['current_block'] = self.current_block
-            res['current_algorithm'] = self.current_algorithm
+            res['current_workspace'] = self.current_workspace
 
         return res
 
-    def set_next_step(self, block_id):
+    def set_next_step(self, block_id=None):
         if self.debug_mode == 'step':
-            if block_id == self.current_block and self.current_algorithm == self.algorithm:
+            if block_id is None or (block_id == self.current_block and self.current_workspace == self.workspace_name):
                 self.is_next_step = True
 
     def set_step(self, block_id):
@@ -127,6 +97,7 @@ class Context(ContextProps):
             # if self.executor.current_block != self.block_id:
             self.is_next_step = False
             self.current_block = block_id
+            self.current_workspace = self.workspace_name
 
     @staticmethod
     def get_child_context(block_context):
@@ -144,11 +115,8 @@ class Context(ContextProps):
 
     def copy(self):
         _self = Context()
-        # _self.params = self.params
-        _self.operation = self.operation
         _self.deferred = self.deferred
         _self.block_context = Helper.copy_via_json(self.block_context)
-        # _self._init_from_dict(copy_via_json(self.to_dict()))
         return _self
 
     def add_deferred(self, deferred_exception):
@@ -179,3 +147,23 @@ class Context(ContextProps):
 
     def set_command_limit(self, debug_mode=None):
         self.limit_commands = 1 if debug_mode else 25
+
+    def variable_scope_add(self, block_context, params):
+        if '_variable_scope' in block_context:
+            self.current_variable_scope = block_context['_variable_scope']
+        else:
+            self.variable_scopes.append(params)
+            self.current_variable_scope = len(self.variable_scopes) - 1
+            block_context['_variable_scope'] = self.current_variable_scope
+
+    def variable_scope_remove(self, block_context):
+        if block_context['_variable_scope'] != self.current_variable_scope \
+                or len(self.variable_scopes) - 1 != self.current_variable_scope:
+            raise ExtException(
+                message='Какая то фигня с variable_scope',
+            )
+        self.variable_scopes.pop(self.current_variable_scope)
+        self.current_variable_scope -= 1
+
+    async def workspace_read(self):
+        raise NotImplementedError('Context.workspace_read')
