@@ -1,12 +1,23 @@
-class LimitCommand(Exception):
+from blockly_executor import ExtException
+
+
+class ServiceException(Exception):
     pass
 
 
-class WorkspaceNotFound(Exception):
+class LimitCommand(ServiceException):
     pass
 
 
-class DeferredOperation(Exception):
+class WorkspaceNotFound(ServiceException):
+    pass
+
+
+class ErrorInBlock(ExtException):
+    pass
+
+
+class DeferredOperation(ServiceException):
     # def __init__(self, name, params, uuid, context):
     #     super(DeferredOperation, self).__init__(name)
     #     self.name =
@@ -21,7 +32,7 @@ class DeferredOperation(Exception):
         }]
 
 
-class StepForward(Exception):
+class StepForward(ServiceException):
     pass
 
     def get_context(self):
@@ -31,5 +42,5 @@ class StepForward(Exception):
         return [self.args[0], self.args[1]]
 
 
-class ReturnFromFunction(Exception):
+class ReturnFromFunction(ServiceException):
     pass
