@@ -2,58 +2,110 @@ import importlib
 import logging
 import os
 import pkgutil
-import xml.etree.ElementTree as XmlTree
 from typing import Optional
 
 from blockly_executor import Action
 from blockly_executor import ExtException
 from blockly_executor import Helper
-from blockly_executor.core.exceptions import DeferredOperation, StepForward, LimitCommand
-from blockly_executor.plugins.standard_blocks.blocks.procedures_defnoreturn import ProceduresDefnoreturn
-from blockly_executor.plugins.standard_blocks.blocks.procedures_defreturn import ProceduresDefreturn
-from blockly_executor.plugins.standard_blocks.blocks.root import Root
+from blockly_executor.core.exceptions import DeferredOperation, StepForward, LimitCommand, WorkspaceNotFound
+from .workspace import Workspace
 
 
 class BlocklyExecutor:
-    ns = {'b': 'https://developers.google.com/blockly/xml'}
     _blocks_index = None
     _blocks_class = {}
 
-    def __init__(self, *, current_block: str = None, current_algorithm: str = None, logger=None,
-                 plugins: Optional[list] = None, breakpoints: Optional[dict] = None,
-                 debug_mode: str = None, **kwargs):
+    def __init__(self, *, logger=None, plugins: Optional[list] = None, breakpoints: Optional[dict] = None, **kwargs):
         """
-        :param current_block: идентификатор блока который сейчас исполняется
-        :param current_algorithm: имя схемы которая в настоящий момент исполняется
         :param logger: класс обеспечивающий вывод лога
         :param plugins: список модулей содержащих реализацию блоков
         :param breakpoints: список идентификаторов блоков на которых требуется остановиться в режиме отладки -
                 словарь в ключе имя workspace для которого точки, в значении массив с идентификаторами блоков
-        :param debug: режим отладки, исполнение идет до следующей точки останова
-        :param step_by_step: режим пошагового исполнения
         :param kwargs:
         """
         # self.current_block = current_block
         # self.current_algorithm = current_algorithm
         # self.current_thread = None
         self.breakpoints = breakpoints
-        self.debug_mode = debug_mode
         self.logger = logger if logger else logging.getLogger(self.__class__.__name__)
 
         self.extend_plugins = plugins
 
-        self.functions = None
-        self.variables = None
-        self.root = None
-
         self.multi_thread_mode = False
 
         self.gather = []
 
         self.commands_result = {}
-        self.algorithm = None
 
         self.action: Optional[Action] = None
+        self.workspace: Optional[Workspace] = None
+
+    async def execute(self, context, *, endpoint=None, commands_result=None):
+        workspace_raw = await context.workspace_read()
+        self.workspace = self.init_workspace(workspace_raw, context.workspace_name, self.logger)
+        context.status = 'run'
+        context.set_command_limit(context.debug_mode)
+        self.action = Action('BlocklyExecutor.execute')
+
+        start_block = self.workspace.get_start_block(endpoint, context)
+
+        try:
+            self._index_commands_result(context, commands_result)
+            context.commands = []
+
+            if context.deferred:
+                await self._execute_deferred(start_block, context)
+            context.check_command_limit()
+
+            self.logger.debug('')
+            self.logger.debug('--------execute----------------------------')
+            self.logger.debug(
+                f'deferred:{len(context.deferred)}, '
+                f'commands:{len(context.commands)}, '
+                f'result:{len(self.commands_result.keys())}')
+
+            result = await start_block.execute(start_block.node, '', context, context.block_context)
+
+            context.result = result
+            self.logger.debug('Complete')
+            context.status = 'complete'
+        except (DeferredOperation, LimitCommand) as err:
+            self.logger.debug(f'raise {err.__class__.__name__}')
+            pass
+        except WorkspaceNotFound as err:
+            context.status = 'error'
+            context.result = dict(message='WorkspaceNotFound', detail=str(err))
+            pass
+        except StepForward as step:
+            context.result = step.args[2]  # block_context
+            context.current_variables = step.args[1].variables
+            context.current_block = step.args[0]
+            context.current_workspace = step.args[3]
+            self.logger.debug('raise StepForward')
+        except ExtException as err:
+            context.status = 'error'
+            context.result = err.to_dict()
+        except Exception as err:
+            context.status = 'error'
+            error = ExtException(parent=err, skip_traceback=-2)
+            context.result = error.to_dict()
+        # context.commands = self.commands
+        # context.deferred = self.gather
+
+        self.logger.debug(
+            f'commands {len(context.commands)} command'
+            f'gather:{len(self.gather)}, '
+            f'result:{len(self.commands_result)}')
+        # self.logger.block_context(f'------------------')
+        self.action.set_end()
+        return context
+
+    async def execute_nested(self, context, *, endpoint=None, commands_result=None):
+        workspace_raw = await context.workspace_read()
+        self.workspace = self.init_workspace(workspace_raw, context.workspace_name, self.logger)
+        start_block = self.workspace.get_start_block(endpoint, context)
+
+        return await start_block.execute(start_block.node, '', context, context.block_context)
 
     @property
     def blocks_index(self) -> dict:
@@ -102,96 +154,6 @@ class BlocklyExecutor:
         except Exception as err:
             raise ExtException(message='Block handler not found', detail=block_name, parent=err)
 
-    @staticmethod
-    def workspace_to_tree(workspace_xml):
-        XmlTree.register_namespace('', 'https://developers.google.com/blockly/xml')
-        return XmlTree.fromstring(workspace_xml)
-
-    def _init_start_block(self, workspace_xml, endpoint, context):
-        # стартуем с функции
-        self.root = self.workspace_to_tree(workspace_xml)
-        self.read_procedures_and_functions()
-        self.variables = self.read_variables()
-        if endpoint:
-            try:
-                block = self.functions[endpoint]
-            except KeyError:
-                context.status = 'error'
-                context.result = f'not found endpoint {endpoint}'
-                return context
-        else:
-            try:
-                block = self.functions['main']
-            except KeyError:
-                block = Root.init(self, '', self.root, logger=self.logger)
-        return block
-
-    async def execute_nested(self, workspace_xml, context, *, endpoint=None, commands_result=None, algorithm=None):
-        self._check_workspace(workspace_xml, algorithm)
-        self.algorithm = algorithm
-        # self.commands_result = commands_result
-        start_block = self._init_start_block(workspace_xml, endpoint, context)
-
-        return await start_block.execute(start_block.node, '', context, context.block_context)
-
-    async def execute(self, workspace_xml, context, *, endpoint=None, commands_result=None, algorithm=None):
-        self._check_workspace(workspace_xml, algorithm)
-        self.algorithm = algorithm
-        context.status = 'run'
-        context.set_command_limit(self.debug_mode)
-        self.action = Action('BlocklyExecutor.execute')
-
-        start_block = self._init_start_block(workspace_xml, endpoint, context)
-
-        try:
-            self._index_commands_result(context, commands_result)
-            context.commands = []
-
-            if context.deferred:
-                await self._execute_deferred(start_block, context)
-            context.check_command_limit()
-
-            self.logger.debug('')
-            self.logger.debug('--------execute----------------------------')
-            self.logger.debug(
-                f'deferred:{len(context.deferred)}, '
-                f'commands:{len(context.commands)}, '
-                f'result:{len(self.commands_result.keys())}')
-
-            result = await start_block.execute(start_block.node, '', context, context.block_context)
-            context.result = result
-            self.logger.debug('Complete')
-            context.status = 'complete'
-        except DeferredOperation:
-            self.logger.debug('raise DeferredOperation')
-            pass
-        except LimitCommand:
-            self.logger.debug('raise LimitCommand')
-            pass
-        except StepForward as step:
-            context.result = step.args[2]
-            context.current_variables = step.args[1].variables
-            context.current_block = step.args[0]
-            context.current_algorithm = step.args[3]
-            self.logger.debug('raise StepForward')
-        except ExtException as err:
-            context.status = 'error'
-            context.result = err.to_dict()
-        except Exception as err:
-            context.status = 'error'
-            error = ExtException(parent=err, skip_traceback=-2)
-            context.result = error.to_dict()
-        # context.commands = self.commands
-        # context.deferred = self.gather
-
-        self.logger.debug(
-            f'commands {len(context.commands)} command'
-            f'gather:{len(self.gather)}, '
-            f'result:{len(self.commands_result)}')
-        # self.logger.block_context(f'------------------')
-        self.action.set_end()
-        return context
-
     async def _execute_deferred(self, robot, context):
         self.logger.block_context('')
         self.logger.block_context('--------execute deferred--------------')
@@ -219,35 +181,24 @@ class BlocklyExecutor:
         for elem in _delete:
             context.deferred.pop(elem)
 
-    def read_procedures_and_functions(self):
-        self.functions = {}
-        for node in self.root.findall("./b:block[@type='procedures_defreturn']", self.ns):
-            name = node.find("./b:field[@name='NAME']", self.ns).text
-            self.functions[name] = ProceduresDefreturn.init(self, name, node, logger=self.logger)
-
-        for node in self.root.findall("./b:block[@type='procedures_defnoreturn']", self.ns):
-            name = node.find("./b:field[@name='NAME']", self.ns).text
-            self.functions[name] = ProceduresDefnoreturn.init(self, name, node, logger=self.logger)
-
-    def read_variables(self):
-        result = {}
-        for node in self.root.findall("./b:variables/b:variable", self.ns):
-            name = node.text
-            result[name] = None
-        return result
-
     @property
     def current_algorithm_breakpoints(self):
         try:
-            return self.breakpoints[self.algorithm]
+            return self.breakpoints[self.workspace.name]
         except (KeyError, TypeError):
             return None
 
-    @staticmethod
-    def _check_workspace(workspace_xml, algorithm):
-        if not workspace_xml:
+    def init_workspace(self, workspace_raw, algorithm, logger):
+        if not workspace_raw:
             raise ExtException(message='Передан пустой алгоритм', detail=algorithm)
 
+        if isinstance(workspace_raw, dict):
+            from .workspace_json import WorkspaceJson
+            return WorkspaceJson(self, workspace_raw, algorithm, logger)
+        elif isinstance(workspace_raw, str):
+            from .workspace_xml import WorkspaceXml
+            return WorkspaceXml(self, workspace_raw, algorithm, logger)
+
     def _index_commands_result(self, context, commands_result):
         if commands_result:
             for command in commands_result:
@@ -259,4 +210,3 @@ class BlocklyExecutor:
             command_uuid = command.get('uuid')
             if command_uuid and command_uuid not in self.commands_result:
                 raise ExtException(message='Не получен ответ на команду', detail=command_uuid)
-
