@@ -43,5 +43,5 @@ class Format(SimpleBlock):
             operation = operations[block_context['type']]
         except KeyError as key:
             raise NotImplemented(f'block {self.__class__.__name__} operation {block_context["type"]} ')
-        block_context['result'] = operation(block_context['value'], block_context['template'])
-        return block_context['result']
+        block_context[self._result] = operation(block_context['value'], block_context['template'])
+        return block_context[self._result]
